/* 
 * gselect - Allow user to interactively select one line from input
 *
 * Copyright (C) 1997, 1998 Ren Seindal
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

char *selection = NULL;			/* The result */

/* 
 * Find a possible selection in LIST, print it and exit.  WIDGET is
 * ignored (it is the ok_button from create_window() below).
 */
void
print_selection(GtkWidget *widget, GtkWidget *list)
{
    GList *s;

    g_return_if_fail(list != NULL);
    g_return_if_fail(GTK_IS_LIST(list));

    s = GTK_LIST(list)->selection;

    if (s != NULL) {
	g_return_if_fail(GTK_IS_BIN(s->data));
	g_return_if_fail(GTK_IS_LABEL(GTK_BIN(s->data)->child));

	selection = strdup(GTK_LABEL(GTK_BIN(s->data)->child)->label);
    }

    gtk_main_quit();
}


/* 
 * Called by button_press in the LIST, where EVENT indicates the
 * GtkListItem clicked, and USER_DATA is a pointer to the "Select"
 * button. 
 */
gint
list_clicked(GtkWidget *list, GdkEventButton *event, gpointer user_data)
{
    GtkWidget *ok;
    GtkWidget *event_widget;

    g_return_val_if_fail(list != NULL, FALSE);
    g_return_val_if_fail(event != NULL, FALSE);

    ok = GTK_WIDGET(user_data);
    g_return_val_if_fail(ok != NULL, FALSE);
    g_return_val_if_fail(GTK_IS_BUTTON(ok), FALSE);

    event_widget = gtk_get_event_widget((GdkEvent*)event);
    if (GTK_IS_LIST_ITEM(event_widget) && 
	(event->type == GDK_2BUTTON_PRESS 
	 || event->type == GDK_3BUTTON_PRESS)) {
	gtk_list_select_child(GTK_LIST(list), event_widget);
	gtk_button_clicked(GTK_BUTTON(ok));

#ifdef DEBUG
	;{
	    GList *s;
	    char *selection = "NONE";

	    s = GTK_LIST(list)->selection;

	    if (s != NULL) {
		selection = strdup(GTK_LABEL(GTK_BIN(s->data)->child)->label);
	    }
	    printf("Multi click %s\n", selection);

	}
#endif /* DEBUG */
    }

    return TRUE;
}

/* 
 * Read the file FP and add each line as a item to the LIST.
 */
int
read_lines(GtkWidget *list, FILE *fp)
{
    char buf[10240];
    GtkWidget *item;

    while (fgets(buf, sizeof buf, fp) != NULL) {
	buf[strlen(buf)-1] = '\0';

	item = gtk_list_item_new_with_label(buf);
	gtk_container_add(GTK_CONTAINER(list), item);
	gtk_widget_show(item);

#ifdef _DEBUG
	printf("Stored \"%s\"\n", buf);
#endif
    }

    return TRUE;
}

/* 
 * Create widget hierarchy as follows:
 *
 * window (with TITLE)
 *   vbox
 *     scrolled_window
 *       list
 *         list_item* (read from FP)
 *     hseparator
 *     alignment
 *       hbox
 *         ok_button
 *         cancel_button
 */

int
create_window(char *title, FILE *fp)
{
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *scrwin;
    GtkWidget *list;
    GtkWidget *hbox;
    GtkWidget *ok_button;
    GtkWidget *cancel_button;
    GtkWidget *tmp;
    GtkAccelGroup *acctable;
    GtkRequisition req;
    guint width;
    guint height;

    acctable = gtk_accel_group_new();

    /* Create main window */
    window = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
    gtk_window_set_title(GTK_WINDOW(window), title);
    gtk_container_border_width(GTK_CONTAINER(window), 10);


    /* Create vbox to contain the rest (could I use window directly?) */
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    gtk_widget_show(vbox);


    /* Create scrolled_window to have automatic scrollbars added */
    scrwin = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
				   GTK_POLICY_AUTOMATIC, 
				   GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox), scrwin, TRUE, TRUE, 0);
    gtk_widget_show(scrwin);


    /* Grab size of scrolled_window to guess size of scrollbars */
    gtk_widget_size_request(GTK_WIDGET(scrwin), &req);
#ifdef DEBUG
    printf("scrwin\t\tw=%5d, h=%5d\n", req.width, req.height);
#endif
    width = req.width/2;		/* kludge */
    height = req.height/2;		/* likewise */


    /* Create and fill list of possible selections */
    list = gtk_list_new();
    gtk_list_set_selection_mode(GTK_LIST(list), GTK_SELECTION_SINGLE);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrwin), list);
    read_lines(list, fp);
    gtk_widget_show(list);

    /* Get size of list and add to estimated size*/
    gtk_widget_size_request(GTK_WIDGET(list), &req);
#ifdef DEBUG
    printf("list\t\tw=%5d, h=%5d\n", req.width, req.height);
#endif

    width += req.width;
    height += req.height;


    /* restrict to max 80%x80% of screen area */
    if ((gdk_screen_width()*8/10) < width)
	width = gdk_screen_width()*8/10;
    if ((gdk_screen_height()*8/10) < height)
	height = gdk_screen_height()*8/10;

#ifdef DEBUG
    printf("desired\t\tw=%5d, h=%5d\n", width, height);
#endif
    gtk_widget_set_usize(GTK_WIDGET(scrwin), width, height);


    /* Create a horizontal separator */
    tmp = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, TRUE, 0);
    gtk_widget_show(tmp);


    /* This seems keep the buttons better in place */
    tmp = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 0);
    gtk_widget_show(tmp);


    /* Create hbox for alignment of buttons  */
    hbox = gtk_hbox_new(TRUE, 10);
    gtk_container_add(GTK_CONTAINER(tmp), hbox);
    gtk_widget_show(hbox);


    /* Create button for selecting - marked as default (return activates) */
    ok_button = gtk_button_new_with_label("Select");
    gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
			GTK_SIGNAL_FUNC(print_selection), list);
    gtk_box_pack_start(GTK_BOX(hbox), ok_button, FALSE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default(ok_button);
    gtk_widget_show(ok_button);

    gtk_widget_add_accelerator(GTK_WIDGET(ok_button), "clicked",
				   acctable, '\n', 0, GTK_ACCEL_VISIBLE);

    /* Create button for canceling - quits program */
    cancel_button = gtk_button_new_with_label("Cancel");
    gtk_signal_connect_object(GTK_OBJECT(cancel_button), "clicked",
			       GTK_SIGNAL_FUNC(gtk_main_quit),
			       GTK_OBJECT(window));
    gtk_box_pack_start(GTK_BOX(hbox), cancel_button, FALSE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
    gtk_widget_show(cancel_button);

    gtk_widget_add_accelerator(GTK_WIDGET(cancel_button), "clicked",
			       acctable, 'q', 0, GTK_ACCEL_VISIBLE);
    gtk_widget_add_accelerator(GTK_WIDGET(cancel_button), "clicked",
			       acctable, '\e', 0, GTK_ACCEL_VISIBLE);


    /* Catch button_press to see double click in list */

    gtk_signal_connect(GTK_OBJECT(list), 
		       "button_press_event",
		       GTK_SIGNAL_FUNC(list_clicked),
		       ok_button);



    /* Center the window on screen */
    gtk_widget_size_request(GTK_WIDGET(window), &req);
    gtk_widget_set_uposition(GTK_WIDGET(window),
			     (gdk_screen_width()-req.width)/2,
			     (gdk_screen_height()-req.height)/2);

    gtk_window_add_accel_group(GTK_WINDOW(window), acctable);

    /* Show it */
    gtk_widget_show(window);

    return 0;
}

void read_rc(char *rc)
{
    GString *s = g_string_new((char *)getenv("HOME"));
    g_string_append_c(s, '/');
    g_string_append(s, rc);

    gtk_rc_parse(s->str);
}

int
main(int argc, char **argv)
{
    int excode = 1;

    gtk_init(&argc, &argv);
    read_rc(".gtkrc");

    create_window("Selector", stdin);

    gtk_main();

    if (selection != NULL) {
	puts(selection);
	excode = 0;
    }

    gtk_exit(excode);
    return 0;
}

